@@ -6,9 +6,7 @@
 #include <grp.h>
 #include <termios.h>
 #include <sys/ioctl.h>
-#include "str_defs.h"
-#include "scan_defs.h"
-#include "fmt_defs.h"
+#include "lib.h"
 #define UUU "unable to "
 
 void w(char *s) { write(2, s, str_len(s)); }
@@ -44,14 +42,14 @@ unsigned int expand_string(char *tmp, char *string, char *user, char *tty) {
   }
 
   if (found_one) {
-    if (tmp==0) return pos - tmp;
+    if (tmp==0) return pos - tmp + 1;
     else *pos = 0;
   }
   return 0;
 }
 
-int main(int argc, char **argv, char **env) {
-  char **arg, **aa, *in, *s, *tmp, ch;
+int main(int argc, char **argv) {
+  char **arg=0, **ee, **aa, *in=0, *s, ch;
   char *user, *tty, flagsetsid=0, flagdaemon=0, flagtty=0, *flagpid=0;
   unsigned long n;
   uid_t uid=0;
@@ -67,47 +65,41 @@ int main(int argc, char **argv, char **env) {
     return 100;
   }
   
-  in	= argv[1];
   user	=(argc>2) ? argv[2] : "";
   tty	=(argc>3) ? argv[3] : "";
 
+  do {
+    n = expand_string(in, argv[1], user, tty);
+    if (n==0) break;
+    in = alloca(n);
+  } while (1);
+  if (in==0) in=argv[1];
+
   ch = *in++;	 /* don't use '%' as split char */
-  if (ch == 0 || *in == 0 ||
-      (n=splitmem(0, in, ch)) < 2) goto usage;
-
-  arg = alloca((n+5) * sizeof(char *)); if (arg==0) _exit(111);
-  splitmem(arg, in, ch);
-
-  for (aa=arg; (s=*aa); ++aa) {
-    n = expand_string(0, s, user, tty);
-    if (n==0) continue;
-    tmp = alloca(n + 4); if (tmp==0) _exit(111);
-    *aa = tmp;
-    expand_string(tmp, s, user, tty);
-  }
+  if (ch == 0 || *in == 0) goto usage;
+  GLOBAL_split_plus(arg,in,ch, n,5);
+  if (n < 2) goto usage;
 
   for (aa=arg; (s=*aa); aa++) {
     if (*s != '-') break;
     s += 2;
+    n=x_atoi(s);
     switch (s[-1]) {
     case 'e':
-      if (*s == 0) { env = 0; }
-      else {
-	ch = *s++;
-	n=splitmem(0,s,ch);
-	env=alloca((n+1) * sizeof(char*)); if (env==0) _exit(1);
-	splitmem(env,s,ch);
-      }
+      ee = 0;
+      ch = *s++;
+      if (ch) GLOBAL_split(ee,s,ch, n);
+      environ = ee;
       break;
     case 'D': flagdaemon=1; break;
     case 'S': flagsetsid=1; break;
     case 'N': flagtty=1; break;
     case 'C': flagtty=2; break;
     case 'p': if (*s) flagpid=s; break;
-    case 'g': scan_ulong(s, &n); gid = n; break;
-    case 'u': scan_ulong(s, &n); uid = n; break;
-    case 'a': scan_ulong(s, &n); alarm(n); break;
-    case 's': scan_ulong(s, &n); sleep(n); break;
+    case 'g': gid = n; break;
+    case 'u': uid = n; break;
+    case 'a': alarm(n); break;
+    case 's': sleep(n); break;
     case 'd': if (chdir(s))  { e(s, ": chdir error");  return 100; } break;
     case 'r': if (chroot(s)) { e(s, ": chroot error"); return 100; } break;
     case '-': aa++; goto do_it;
@@ -131,7 +123,7 @@ int main(int argc, char **argv, char **env) {
   if (flagpid) {
     int fd = open(flagpid, O_RDWR | O_TRUNC | O_CREAT, 0644);
     if (fd>=0) {
-      char tmp[12];
+      char tmp[3*sizeof(unsigned long)];
       n=fmt_ulong(tmp, getpid());
       tmp[n] = '\n';
       write(fd, tmp, n+1);
@@ -150,7 +142,7 @@ int main(int argc, char **argv, char **env) {
   if (s==0 || *aa==0) return 0;
   if (aa[0][0] == 0) aa[0] = s;
 
-  execve(s, aa, env);
+  execve(s, aa, environ);
   e(s, ": exec error");
   return 127;
 }
