@@ -1,25 +1,8 @@
-/*
- * ngetty-helper.c
- *
- * Copyright 2007,2008 Nikola Vladov
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
-
 #define _GNU_SOURCE
 
 #include <unistd.h>
 #include <stdlib.h>
 #include <fcntl.h>
-#include <time.h>
 #include <alloca.h>
 #include <errno.h>
 #include <termios.h>
@@ -99,22 +82,15 @@ static void output_special_char(char *s, int len) {
     case 'u':
     case 'U':
       {
-	int fd=open(_PATH_UTMP, O_RDONLY);
-	if (fd >= 0) {
-	  struct utmp ut;
-	  for (len=0; utmp_io(fd, &ut, F_RDLCK); )
-	    if (ut.ut_type == USER_PROCESS) len++;
-	  close(fd);
-	  
-	  x = (char *)&ut;
-	  x += fmt_ulong(x, len);
-	  if (c=='U') {
-	    x += str_copynz(x, " users", 10);
-	    if (len==1) --x;
-	  }
-	  *x = 0;
-	  outs((char *)&uts);
+	char ubuf[8 + 3*sizeof(unsigned long)], *x = ubuf;
+	unsigned long usrs = utmp_users();
+	x += fmt_ulong(x, usrs);
+	if (c=='U') {
+	  x += str_copynz(x, " users", 10);
+	  if (usrs==1) --x;
 	}
+	*x = 0;
+	outs(ubuf);
       }
       break;
 #endif
@@ -251,17 +227,15 @@ int main(int argc,char *argv[]) {
   if (!str_diffn(tty, "/dev/", 5)) tty += 5;
 
   if ((fd = open(NGETTY_HOME "/Conf", O_RDONLY)) >=0) {
-    char **aa, *q;
+    char **aa=0, *q;
     int tty_len = str_len(tty);
     /* 40 bytes for Months or Days */
     if (GLOBAL_fstat_READ(fd,st,s, len,64000, 40)) _exit(1);
     close(fd);
     if (st.st_uid || st.st_gid || (st.st_mode & 07177)) len = 0;
     s[len]=0;
-    
-    len = splitmem(0,s,'\n');
-    aa=alloca((len+1) * sizeof(char*)); if (aa==0) _exit(1);
-    splitmem(aa,s,'\n');
+
+    GLOBAL_split(aa,s,'\n', len);
 
     for (; *aa; aa++) {
       s=aa[0];
@@ -286,13 +260,9 @@ int main(int argc,char *argv[]) {
   if ((s=o[Onice])) nice(x_atoi(s));
 
   if ((s=o[Oenviron])) {
-    char ch=*s++;
-    if (ch) {
-      len=splitmem(0,s,ch);
-      environ=alloca((len+1) * sizeof(char*)); if (environ==0) _exit(1);
-      splitmem(environ,s,ch);
-    } else
-      environ=0;
+    char **ee=0, ch=*s++;
+    if (ch) GLOBAL_split(ee,s,ch, len);
+    environ=ee;
   }
 
   if (argc>2 && argv[2][0] == 'l') { /* get logname and execve() login */
