@@ -1,11 +1,9 @@
 /* nwho.c */
 #include <unistd.h>
-#include <time.h>
 #include <fcntl.h>
-#include <utmp.h>
 #include <errno.h>
 #include <sys/wait.h>
-#include "all_defs.h"
+#include "lib.h"
 
 #define S(a,b)		str_copy(a,b)
 #define SN(a,b,c)	str_copynz(a,b,c)
@@ -18,7 +16,6 @@ void usage(char *buf) {
 }
 
 int main(int argc, char **argv) {
-  struct utmp ut[1];
   struct tm *tm;
   struct stat st;
   char *montab = "JanFebMarAprMayJunJulAugSepOctNovDec";
@@ -30,7 +27,12 @@ int main(int argc, char **argv) {
   char flagcheck = 1;
   char flagup = 0;
   char flagmesg = 0;
+#ifdef USE_LIBC_UTMP
+  struct utmp_type *ut;
+#else
+  struct utmp_type ut[1];
   int fd;
+#endif
 
   argc--; argv++;
   while (argc>0) {
@@ -49,13 +51,18 @@ int main(int argc, char **argv) {
     argc--; argv++;
   }
 
+#ifdef USE_LIBC_UTMP
+  if (argc>0)  f_utmpname(argv[0]);
+  setutent();
+#else
   if (argc>0)  p = argv[0];
-  else p = _PATH_UTMP;
+  else p = Utmp_File;
 
   fd = open(p, O_RDONLY);
   if (fd == -1) {write(1,"error open input\n",17); _exit(1);}
+#endif
 
-  while (utmp_io(fd, ut, F_RDLCK)) {
+  while (f_getutent()) {
     if (ut->ut_type != USER_PROCESS) continue;
     if (flagcheck)
       if (kill(ut->ut_pid, 0) && errno == ESRCH) continue;
@@ -68,7 +75,7 @@ int main(int argc, char **argv) {
     id = -1;
 
     S(buf, "/dev/");
-    buf[5 + SN(buf+5, ut->ut_line, UT_LINESIZE)] = '\0';
+    buf[5 + SN(buf+5, ut->ut_line, sizeof(ut->ut_line))] = '\0';
 
     if (!stat(buf, &st)) {
       if ((st.st_mode & 0020) == 0020) mesg = '+';
@@ -107,19 +114,11 @@ int main(int argc, char **argv) {
     }
 
     *p++ = '(';
-    if (ut->ut_addr) {
-      unsigned char *z = (unsigned char *)&ut->ut_addr;
-      int k;
-      ipasc = p;
-      for (k=0; k<4; k++) {
-	p += fmt_ulong(p, (unsigned long)z[k]);
-	*p++ = '.';
-      }
-      *--p = 0;
-    } else ipasc = "";
+    ipasc = p;
+    p += fmt_utmp_ip(p, (char *)ut->ut_addr_v6);
 
     if (flagaddr>1 ||
-	(flagaddr && str_diffn(ipasc, ut->ut_host, 20))) {
+	(flagaddr && str_diffn(ipasc, ut->ut_host, sizeof(ut->ut_host)))) {
       *p++ = '=';
       p += SN(p,ut->ut_host,90);
     }
@@ -127,6 +126,6 @@ int main(int argc, char **argv) {
 
     write(1,buf, p-buf);
   }
-  close(fd);
+  f_endutent();
   return 0;
 }
