@@ -4,7 +4,7 @@
 #include "lib.h"
 
 /* type:  F_RDLCK or F_WRLCK */
-struct utmp *utmp_io(int fd, struct utmp *ut, int type) /*EXTRACT_INCL*/ {
+struct utmp_type *utmp_io(int fd, struct utmp_type *ut, int type) /*EXTRACT_INCL*/ {
   struct flock fl;
   int len;
   
